using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Services;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Commands.UpdateVehicleEnrollment;

public class UpdateVehicleEnrollmentCommandAuthorizer :
    AbstractRequestAuthorizer<UpdateVehicleEnrollmentCommand>
{
    private readonly SessionUserService _sessionUserService;
    private readonly UnitOfWork _unitOfWork;

    public UpdateVehicleEnrollmentCommandAuthorizer(
        SessionUserService sessionUserService,
        UnitOfWork unitOfWork)
    {
        _sessionUserService = sessionUserService;
        _unitOfWork = unitOfWork;
    }

    public override void BuildPolicy(UpdateVehicleEnrollmentCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated = _sessionUserService.IsAuthenticated
        });

        var vehicleEnrollment = _unitOfWork.VehicleEnrollmentRepository
            .GetOneAsync(
                e => e.Guid == request.Guid, e => e.Vehicle.Company.Account,
                CancellationToken.None)
            .Result;

        var employees = _unitOfWork.EmployeeRepository
            .GetPageAsync(
                e => request.EmployeeGuids.Contains(e.Guid),
                e => e.Company.Account,
                1, request.EmployeeGuids.Count, CancellationToken.None)
            .Result.Items;

        UseRequirement(new MustBeObjectOwnerOrAdminRequirement
        {
            UserRoles = _sessionUserService.Roles,
            RequiredGuid = vehicleEnrollment?.Vehicle.Company.Account.Guid,
            UserGuid = _sessionUserService.Guid
        });

        foreach (var employee in employees)
        {
            UseRequirement(new MustBeObjectOwnerOrAdminRequirement
            {
                UserRoles = _sessionUserService.Roles,
                RequiredGuid = employee.Company.Account.Guid,
                UserGuid = _sessionUserService.Guid
            });
        }
    }
}
